/* This file is part of GNU Libraries and Engines for Games  -*- c++ -*-

   $Id: condmutex.h,v 1.2 2004/03/03 03:50:02 jechk Exp $
   $Log: condmutex.h,v $
   Revision 1.2  2004/03/03 03:50:02  jechk
   Changed some names, comments and other things for consistency.

   Revision 1.1  2004/03/03 02:05:22  jechk
   Merged many changes.  See ChangeLog for details.


   Created 01/23/04 by Jean-Dominique Frattini <zionarea@free.fr>
   
   Copyright (c) 2004 Free Software Foundation
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
/*! \file support/threads/condmutex.h
  \brief Condition mutex support for threads.
*/

#ifndef LEG_SUPPORT_THREADS_CONDMUTEX_H
#define LEG_SUPPORT_THREADS_CONDMUTEX_H

#include "cond.h"

namespace leg
{
namespace support
{
namespace threads
{
   //! Conditional mutex.
   /*!
    * Conditions under Posix threads stands generally with mutex. So, this class
    * is best for this work.
    *
    * status:
    *	 working.
    *
    * info:
    *	 none.
    *
    * @sa Condition,Mutex,Thread.
    */
   class CondMutex: public Mutex,Condition
   {
      public:

      //! Default constructor.
      CondMutex ();

      //! Copy constructor.
      /*!
       * TODO: test it !
       */
      CondMutex (const CondMutex& mut);
      
      ~CondMutex ();

      //! Affectation.
      /*!
       * TODO: test it !
       */
      CondMutex&
      operator= (CondMutex& src);

      //! Send the condition signal.
      /*!
       * Send the condition signal to the thread that is waiting for it.
       *
       * @sa Wait().
       */
      inline int 
      Signal ()
      {
	 return Condition::Signal ();
      }

      //! Wait for the signal to be send.
      /*!
       * This must be another thread than the one that send the signal that has
       * to wait for.
       *
       * @sa Signal.
       */
      inline int 
      Wait ()
      {
	 return Condition::Wait (static_cast<Mutex&> (*this));
      }

      //! Wait for the signal until the given date.
      inline int
      TimedWait (const timespec *date = 0)
      {
	 return Condition::TimedWait (static_cast<Mutex&> (*this), date);
      }
   };
}// namespace threads
}// namespace support
}// namepsace leg
#endif

